#include "controller.h"
#include "xparameters.h"
#include "ascii_characters.h"
#include "xio.h"
#include "stdio.h"
#include "xuartlite_l.h"
#include "xuartlite.h"

#include "w3_userio.h"
#include "w3_ad_controller.h"
#include "w3_clock_controller.h"
#include "w3_iic_eeprom.h"
#include "radio_controller.h"

//Abstracts away the long xparameters name
//This way, if we change the name of the core or register,
//we only have to change this line.
#define RC_BASEADDR 		XPAR_RADIO_CONTROLLER_0_BASEADDR
#define AD_BASEADDR 		XPAR_W3_AD_CONTROLLER_0_BASEADDR
#define CLK_BASEADDR 		XPAR_W3_CLOCK_CONTROLLER_0_BASEADDR

//Define you adresses here (hint: look them up in xparams.h)


//Instantiates an instance of the UART driver (controls the serial)
static XUartLite UartLite;

//This is going to store our two gain parameters
unsigned char RxCoarseGain = 2; //range is 1 to 3
unsigned int  RxFineGain = 0x10; // range is 0 to 31 (0 to 0x1F)
unsigned int  TxGain = 0x3F; // range is 0 to 63

int main(){
	w3_node_init();

	// Write some code here that gives a user instructions for controlling your transceiver. 
	// --- Enable/disable tranmitter
	// --- Enable/disable receiver
	// --- Enable/disable/reset CFO Correction
	// --- Enable/disable/reset timing correction
	// --- Select output of DAC (to view signals at various stages in your design)
	// --- Modify all filter coefficients (both for CFO and timing synchronization) for tuning


	//Set up the UART
	XUartLite_Initialize(&UartLite, XPAR_UARTLITE_0_DEVICE_ID);

	//Set up the Radio - For details about these calls, see http://warp.rice.edu/svn/WARP/PlatformSupport/CustomPeripherals/pcores/radio_controller_v3_00_b/doc/html/api/index.html
	
	// Configure TX enable, RX enable, and Rx HP filter for software control
	radio_controller_setCtrlSource(RC_BASEADDR, RC_RFA, (RC_REG0_TXEN_CTRLSRC|RC_REG0_RXEN_CTRLSRC|RC_REG0_RXHP_CTRLSRC), RC_CTRLSRC_REG);

	// Configure TX  low-pass filter response to have a corner frequency of 18 MHz:
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_TXLPF_BW, 0x02);

	// Configure TX  low-pass filter response to have a corner frequency of 18 MHz:
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_TXLPF_BW, 0x02);

	//Enable software Tx Gain control:
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA,RC_PARAMID_TXGAINS_SPI_CTRL_EN, 0x01);

	//Enable software Rx Gain control:
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA,RC_PARAMID_RXGAINS_SPI_CTRL_EN, 0x01);

	//Initialize the baseband Tx gain to its max value (0 dB)
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA,RC_PARAMID_TXGAIN_BB, 0x00);

	//Initialize the RF Tx gain
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_TXGAIN_RF, TxGain);

	// Configure receive HPF cutoff of 30kHz (DC block)
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_RXHPF_HIGH_CUTOFF_EN, 0x01);

	//14MHz Low Pass Filter on RX (our max freq is 10MHz + 2.25MHz)
	// 0: 7.5MHz<br>1: 9.5MHz<br>2: 14MHz<br>3: 18MHz
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_RXLPF_BW, 0x03);

	//Initialize the Rx RF Gain (1:0dB, 2:15dB , 3:30dB )
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA,RC_PARAMID_RXGAIN_RF, RxCoarseGain);

	//Initialize the Rx baseband Gain
	radio_controller_setRadioParam(RC_BASEADDR, RC_RFA, RC_PARAMID_RXGAIN_BB, RxFineGain);

	// Enable the receiver by default
	radio_controller_RxEnable(RC_BASEADDR, RC_RFA);

	// Set center frequency (i.e. choose the WiFi channel)
	radio_controller_setCenterFrequency(RC_BASEADDR , RC_RFA, RC_24GHZ, 6);

	while(1){
		//Changes to Configuration via UART
		RecvBuffer = 0;

		//Receives a character from the UART and sticks it in RecvBuffer
		RecvBuffer = XUartLite_RecvByte(STDIN_BASEADDRESS);

		// Here is where you will put conditional statements to respond to user control input

	}
	return 0;
}

// Do NOT modify the code below
int w3_node_init() {

	int status;
	int ret = XST_SUCCESS;

	microblaze_enable_exceptions();

	//Initialize the AD9512 clock buffers (RF reference and sampling clocks)
	status = clk_init(CLK_BASEADDR, 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in clk_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the AD9963 ADCs/DACs for on-board RF interfaces
	ad_init(AD_BASEADDR, (RFA_AD_CS | RFB_AD_CS), 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in ad_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the radio_controller core and MAX2829 transceivers for on-board RF interfaces
	status = radio_controller_init(RC_BASEADDR, (RC_RFA | RC_RFB), 1, 1);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in radioController_initialize (%d)\n", status);
		ret = XST_FAILURE;
	}

	return ret;
}
